from gestionatr.cli import request_p0
def aux():
    def purge_crm_lead(c, cups):
        #import pudb;pu.db
        cups = cups[:20]
        lead_ids = c.GiscedataCrmLead.search([('cups', 'like', cups)])
        if not lead_ids:
            return False, "Sin leads"
        c.GiscedataCrmLead.write(lead_ids, {'state': 'cancel'})
        polissa_ids = [x['polissa_id'][0] for x in c.GiscedataCrmLead.read(lead_ids, ['polissa_id']) if x['polissa_id']]
        if not polissa_ids:
            return True
        can_deactivate_cups = True
        for pid in polissa_ids:
            sw_ids = c.GiscedataSwitching.search([('cups_polissa_id', '=', pid)])
            if sw_ids:
                for sw in c.GiscedataSwitching.browse(sw_ids):
                    if sw.step_id.name == "01" and sw.enviament_pendent:
                        sw.write({'state': 'cancel'})
                        c.GiscedataPolissa.write(pid, {'state': 'cancelada'})
                    elif sw.step_id.name == "02" and sw.rebuig:
                        sw.write({'state': 'cancel'})
                        c.GiscedataPolissa.write(pid, {'state': 'cancelada'})
                    else:
                        can_deactivate_cups = False
        if can_deactivate_cups:
            cups_ids = c.GiscedataCupsPs.search([('name', 'like', cups)])
            if cups_ids:
                c.GiscedataCupsPs.write(cups_ids, {'active': False})
        return True


    P0_DEMO = {
        # Funcionen
        # "Iberdrola": {
        #     'url': "https://www.i-de.es/cnmcws/agentes/sync?wsdl",
        #     'user': "EC980Y4",
        #     'password': "9UWyvGv39D",
        #     'cups': "ES0021000008103774WC0F",
        #     'emisora': "0373",
        #     'destino': "0021",
        # },
        # "Iberdrola Gaba": {
        #     'url': "https://www.i-de.es/cnmcws/agentes/sync",
        #     'user': "EC9800N",
        #     'password': "^LZg$9J0[T",
        #     'cups': "ES0021000007770630PP",
        #     'emisora': "1664",
        #     'destino': "0021",
        # },
        # "Morella": {
        #     'url': "https://ov.maestrazgodistribucion.es/Sync?WSDL",
        #     'user': "0373",
        #     'password': "0373_aeQuee3F",
        #     'cups': "ES0189000038091476YE0F",
        #     'emisora': "0373",
        #     'destino': "0189",
        # },
        # "Endesa": {
        #     'url': "http://trader-eapi.de-c1.eu1.cloudhub.io/api/P0?wsdl",
        #     'user': "ea1f02cb9ed04a1da80496255df63870",
        #     'password': "78415Cd1a3e44798A87d642EF0171517",
        #     'cups': "ES0031300002599001TX0F",
        #     'emisora': "0706",
        #     'destino': "0031",
        # },
        # "Fenosa": {
        #     'url': "https://sctd.gasnaturalfenosa.com/sctd/ws/Sync?wsdl",
        #     'user': "1664WSSAP",
        #     'password': "2022Mayo",
        #     'cups': "ES0022000006704409RB1P",
        #     'emisora': "1664",
        #     'destino': "0022",
        # },
        # "Viesgo": {
        #     'url': "https://viesgop0.app.viesgo.com/syncRequest.wsdl",
        #     'user': "0706",
        #     'password': "ENE190#06",
        #     'cups': "ES0027700037401002ZB0F",
        #     'emisora': "0706",
        #     'destino': "0282",
        # },
        # "ELECTRICA DE LA SERRANIA DE RONDA, S.L": {
        #     'url': "https://intercambioxml.electricaserraniaderonda.com/Sync/Sync.svc",
        #     'user': "0971",
        #     'password': "ef8d6c83-7",
        #     'cups': "ES0225000050200275EL0F",
        #     'emisora': "0971",
        #     'destino': "0225",
        # },
        "Agri distri": {
            'url': "https://clients.agrienergiaelectrica.com/Sync?WSDL",
            'user': "0108",
            'password': "aks!AP2dmmm9",
            'cups': "ES0112000000015671QS0F",
            'emisora': "0108",
            'destino': "0112",
        },
        # # No funcionen encara
        # "Dielsur": {
        #     'url': "http://wsp0dielesur.portalswitching.com/WSCurvas.asmx?wsdl",
        #     'user': "1440",
        #     'password': "Test1440",
        #     'cups': "ES0143000000203855EW0F",
        #     'emisora': "1440",
        #     'destino': "0143",
        # },
        # "CIDE": {
        #     'url': "https://switchingsync.sercide.com/BasicSYNCService.svc",
        #     'user': "0370_i1C6N",
        #     'password': "=mYow<umQV`aq",
        #     'cups': "ES0614000000000035ZJ0F",
        #     'emisora': "0706",view_importacio_linia_general_tree
        #     'destino': "0614",
        # },
    }

    import sys
    for distri in P0_DEMO:
        sys.stdout.write(("Testing {}...".format(distri)))
        res = request_p0(url=P0_DEMO[distri]["url"], user=P0_DEMO[distri]["user"], password=P0_DEMO[distri]["password"], params=P0_DEMO[distri])
        if "CodigoDePaso>02" not in res:
            print "ERROR"
        else:
            print "OK"



# -*- coding: utf-8 -*-
import sys
import click
import requests
import base64
from lxml import etree

from gestionatr.input.messages import message
from gestionatr.input.messages import message_gas
from gestionatr.input.messages.message import except_f1
from gestionatr.output.messages.sw_p0 import ENVELOP_BY_DISTR
from gestionatr.exceptions import *

from gestionatr import __version__


VERSION_TEXT = u'ATR library version (gestionatr) : {0}'.format(__version__)


P0_TEMPLATE = """
        <MensajeSolicitudInformacionAlRegistroDePS xmlns="http://localhost/elegibilidad">
          <Cabecera>
            <CodigoREEEmpresaEmisora>{emisora}</CodigoREEEmpresaEmisora>
            <CodigoREEEmpresaDestino>{destino}</CodigoREEEmpresaDestino>
            <CodigoDelProceso>P0</CodigoDelProceso>
            <CodigoDePaso>01</CodigoDePaso>
            <CodigoDeSolicitud>{solicitud}</CodigoDeSolicitud>
            <SecuencialDeSolicitud>01</SecuencialDeSolicitud>
            <FechaSolicitud>{fecha_solicitud}</FechaSolicitud>
            <CUPS>{cups}</CUPS>
          </Cabecera>
        </MensajeSolicitudInformacionAlRegistroDePS>
"""


def get_gestionatr_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(VERSION_TEXT)
    ctx.exit()


@click.group()
@click.option("--version", "-v", is_flag=True, callback=get_gestionatr_version,
              expose_value=False, is_eager=True, help="ATR library version")
def atr():
    pass


@atr.command(name='test')
@click.option("--filename", "-f", help="path to XML filename", required=True)
@click.option("--sector", "-s", help="e (power) or g (gas)", default="e")
def test(filename, sector):
    with open(filename, 'r') as xml_file:
        try:
            data = xml_file.read()
            if sector == u'e':
                m = message.Message(data)
            elif sector == u'g':
                m = message_gas.MessageGas(data)
            m.parse_xml()
            sys.stdout.write(u'Correct File\n')
        except except_f1 as e:
            error_txt = unicode(e.value).encode(errors='ignore')
            sys.stdout.write(
                u'WARNING: Invalid File: {0}\n'.format(error_txt)
            )
        except Exception as e:
            error_txt = unicode(e).encode(errors='ignore')
            sys.stdout.write(
                u'WARNING: Invalid File: {0}\n'.format(error_txt)
            )
        finally:
            sys.stdout.flush()


def request_p0(url, user, password, xml_str=None, params=None):
    import random
    import re
    from datetime import datetime
    if xml_str is None and params is None:
        raise ValueError("XML or params must be passed to request_p0")
    if xml_str is None and params:
        codi_receptor = params['destino']
        params['fecha_solicitud'] = datetime.now().strftime('%Y-%m-%dT%H:%M:%S')
        params['solicitud'] = 10**11 + int((random.random() * 10**11))
        xml_str = re.sub(r'\s+<', '<', P0_TEMPLATE)
        xml_str = re.sub(r'\s+$', '', xml_str)
        xml_str = re.sub(r'\n', '', xml_str).format(**params)
    else:
        codi_receptor = xml_str.split("<CodigoREEEmpresaDestino>")[1].split("</CodigoREEEmpresaDestino>")[0]

    base64string = base64.encodestring('%s:%s' % (user, password)).replace('\n', '')
    headers = {
        "Authorization": "Basic %s" % base64string,
        "content-type": 'text/xml; charset=utf-8',
    }

    # Clean XML
    xml_str = xml_str.strip()
    xml_str = xml_str.replace("'utf-8'", "'UTF-8'")
    xml_str = xml_str.replace("<?xml version='1.0' encoding='UTF-8'?>", "")

    # Fem sempre 2 intents: amb el que esta definit per aquella distri i si va malament amb una plantilla base que
    # sol funcionar en la majoria. Aixi si una distri no la tenim documentada es fa el intent amb les 2 plantilles
    # principals que tenim. La "altres" i la "reintent"
    distri_envelop = ENVELOP_BY_DISTR.get(codi_receptor, ENVELOP_BY_DISTR.get("altres"))
    retry_envelop = ENVELOP_BY_DISTR.get("reintent")
    error = None
    for envelop in [distri_envelop, retry_envelop]:
        xml_str_to_use = xml_str
        soap_content = envelop['template'].format(xml_str=xml_str_to_use)

        # Send request
        h = headers.copy()
        h.update(envelop['extra_headers'])
        res = requests.post(url, data=soap_content, headers=h, auth=(user, password))
        res = res.content
        try:
            def find_child(element, child_name):
                res = None
                if child_name in element.tag:
                    return element
                for child in element:
                    res = find_child(child, child_name)
                    if res is not None:
                        break
                return res

            aux = etree.fromstring(res)
            aux_res = find_child(aux, "MensajeEnvioInformacionPS")
            if aux_res is None:
                aux_res = find_child(aux, "MensajeRechazoP0")
            if aux_res is None:
                aux_res = find_child(aux, "faultstring")
                if aux_res is not None:
                    error_mssg = {
                        'request': res,
                        'response': etree.tostring(aux_res)
                    }
                    raise P0FaultError(error_mssg)

            res = etree.tostring(aux_res)
            return res
        except P0FaultError as p0efe:
            raise
        except Exception as e:
            if not error:
                error = e

    if error:
        print error

    return res


@atr.command(name='p0')
@click.option('-u', '--url', default='http://localhost', help=u'URL del webservice', show_default=True)
@click.option('-s', '--user', default='admin', help=u'User del webservice', show_default=True)
@click.option('-p', '--password', default='admin', help=u'Password del webservice', show_default=True)
@click.option('-f', '--file', help=u'Fitxer P0 pas 01 per enviar', show_default=True)
@click.option('--emisora', help='Código REE empresa emisora')
@click.option('--destino', help='Código REE empresa destino')
@click.option('--cups', help='CUPS')
def sollicitar_p0(url, user, password, file=None, emisora=None, destino=None, cups=None):
    params = None
    from lxml import etree
    if emisora and destino and cups:
        params = {
            'emisora': emisora,
            'destino': destino,
            'cups': cups
        }

    res = request_p0(url, user, password, file, params)
    res = etree.fromstring(res)
    print(etree.tostring(res, pretty_print=True))


